/*
   $Id: ComparisonFrequency.cpp 648 2011-10-31 12:59:22Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#include <cerrno>
#include <vector>
using namespace std;

#ifdef _OPENMP
#include <omp.h>
#endif

#include "Comparison.h"
#include "Display.h"
#include "DisplayM1.h"
#include "DisplaySam.h"
#include "dust.h"
#include "GlintEngine.h"
#include "QueryBufferDouble.h"
#include "ComparisonFrequency.h"

/**
 *  \brief Start the comparison process
 *         -# Create, use and destroy an object deriving from QueryBuffer
 *         -# Create, use and destroy an object deriving from Display
 *         -# Create, use and destroy an object deriving from GlintEngineBase
 *         NOTE - The glintEngineManager is NOT USED YET
 *         -# Create a master Querybuffer
 *         -# Create a QBufferManager
 *         -# while there are sequences to read, call _Run
 */


void ComparisonFrequency::Run()
{	
	// Create the GlintEngine
	QueryBuffer* query_ptr = _InitQueryBuffer();
	if ( _parameters.IsToUpper() ) QueryBuffer::ToUpper( true );

	// Using an auto_ptr for display, because there is only ONE display object
	auto_ptr<Display>  display_aptr (_InitDisplay());
	auto_ptr<GlintEngine> engine_aptr(NULL);

	engine_aptr.reset(_InitGlintEngine(query_ptr,*display_aptr) );
	
	if ( !_parameters.IsSilent() )
		cerr << "Processing the request\n";
	
	while ( query_ptr -> ReadNextSeq() )
	{
		engine_aptr -> Run();
	}
		
	if ( !_parameters.IsSilent() )
		fprintf(stderr,"Leaving read query\n");
}



/*
void ComparisonFrequency::Run()
{
	// Create the query buffer
	auto_ptr<QueryBuffer> query_aptr( _InitQueryBuffer() );
	if ( _parameters.IsToUpper() ) QueryBuffer::ToUpper( true );

	auto_ptr<Display>         display_aptr (_InitDisplay());
	GlintEngineFreq engine (query_aptr.get(),*display_aptr);

	while ( query_aptr -> ReadNextSeq() )
	{
		engine.Run();
	}
	
	if ( !_parameters.IsSilent() )
		fprintf(stderr,"Leaving read query\n");
}
* */
