/*
   $Id: time.cpp 648 2011-10-31 12:59:22Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#include <iostream>
#include <iomanip>
#include <omp.h>
using namespace std;

#include "time.h"


ThreadsTimeRecorder::ThreadsTimeRecorder()
{
	double now = omp_get_wtime();
	for (int i=0; i<omp_get_max_threads();++i)
	{
		__time_infos.push_back(TimeInfo());
		__time_infos[i].push_back(-now);
	}
}
	
void ThreadsTimeRecorder::Running()
{
	int    tid = omp_get_thread_num();
	double now = omp_get_wtime();
	__time_infos[tid].push_back(now);
}
void ThreadsTimeRecorder::Waiting()
{
	int    tid = omp_get_thread_num();
	double now = omp_get_wtime();
	__time_infos[tid].push_back(-now);
}

#ifdef OLD_STUFF
ostream & operator<<(ostream& os, const ThreadsTimeRecorder& ttr)
{
	os.setf(ios::left,ios::adjustfield);
	os << setw(15) << "tid" <<setw(15)<< "waiting (s)"<< setw(15)<< "running (s)" << endl;
	for (size_t i=0; i<ttr.__time_infos.size(); ++i)
	{
		os << setw(15) <<i << setw(15) <<ttr.__time_infos[i].waiting_duration << setw(15) <<ttr.__time_infos[i].running_duration << endl;
	}
	return os;
}
#endif

ostream & operator<<(ostream& os, const ThreadsTimeRecorder& ttr)
{
	int tid=0;
	for (vector<TimeInfo>::const_iterator i=ttr.__time_infos.begin(); i!=ttr.__time_infos.end(); ++i,++tid)
	{
		const TimeInfo & tinf = *i;
		os.setf(ios::left,ios::adjustfield);
		os << "THREAD nb " << tid << '\n';
		if ( tinf.size() >= 2 )
		{
			for (size_t j=1; j<tinf.size(); ++j)
			{
				os << setw(15) << j << ttr.__Duration(tinf,j) << '\n';
			}
		}
		os << '\n';
	}
	return os;
}

string ThreadsTimeRecorder::__Duration(const TimeInfo& tinf, size_t j) const
{
	ostringstream out;
	if ( tinf[j-1]*tinf[j] > 0 )
	{
		out << "*** ERROR IN SEQUENCE (ThreadsTimeRecorder) ";
		out << "position " << j << " = " << tinf[j] << " previous = " << tinf[j-1] << "\n";
		throw(runtime_error(out.str().c_str()));
	}
	else
	{
		if ( tinf[j] > 0 )
		{
			out << "W ";
			out << setw(15) << tinf[j]+tinf[j-1];
		}
		else
		{
			out << "R ";
			out << setw(15) << -tinf[j]-tinf[j-1];
		}
		return out.str();
	}
}
